#!/usr/bin/env python
#
#       explyline.py
#       
#       Copyright 2010 Chris Hill <>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

# extract a polyline from a shapefile and save it as an OSM file for loading

import sys
import os.path
from optparse import OptionParser
import shpUtils

def main():
    parser = OptionParser(usage="%prog [options]", version="%prog 1.2")
    parser.add_option("-s", "--shp", dest="shpfile",help="the shpfile name to use")
    #parser.add_option("-p", "--polyline", dest="polyline", type="int", default=1,help="the polyline number to extract")
    parser.add_option("-b", "--bounds",action="store_true", dest="bounds", default=False,help="add bounds statement")
    parser.add_option("-q", "--quiet",action="store_false", dest="verbose", default=True,help="don't print status messages to stdout")
    parser.add_option("-n", "--nodeid",action="store", dest="nodeid", type="int",default=-1, help="The first nodeid to use, it should be negative")
    parser.add_option("-w", "--wayid",action="store", dest="wayid", type="int", default=-1, help="The first wayid to use, it should be negative")
    parser.add_option("-i", "--index",action="store", dest="index", type="int", default=0, help="The index of the polyline to extract")
    parser.add_option("-o", "--osm",action="store",dest="osmfile", help="The output osm file name")
    (options, args) = parser.parse_args()
    
    if len(args) != 1 :
        print "You must enter the shapefilename"
        sys.exit(1)
    
    shapefilename=args[0] # the left over stuff when the options have been extracted
    
    if  not os.path.exists(shapefilename) :
        print "The shpfile %s cannot be found" % options.shpfile
        sys.exit(1)
    
    startnodeid=options.nodeid
    if startnodeid>=0 :
        print "Nodeid must be a negative number"
        sys.exit(1)
    
    startwayid=options.wayid
    if startwayid>=0 :
        print "Wayid must be a negative number"
        sys.exit(1)
    
    if options.osmfile is None :
        osmfile="pl%d.osm" % (polyline)
    else :
        osmfile=options.osmfile
    
    polyline=options.index
    
    if options.verbose :
        print "ExPlyLine - extract the polyline with a specific number from a shapefile"
    
    if options.verbose :
        print "Loading the shapefile: "+shapefilename
    # load the shapefile, populating a list of dictionaries
    shpRecords = shpUtils.loadShapefile(shapefilename)
    
    #work through the records, looking for the named polyline
    #for poly in shpRecords :
    #    plname=poly['dbf_data']['NUMBER']
    #    if plname==polyline :
    #        if options.verbose :
    #            print "Found %d." % (plname)
    #        # found the polyline, so start to write the file
    
    poly=shpRecords[polyline]
    pf = file(osmfile,"w")
    pf.write("<?xml version='1.0' encoding='UTF-8'?>\n")
    pf.write("<osm version='0.6' generator='extractpoly'>\n")
    
    # if bounds are requested, write them to the .osm file
    if options.bounds :
        ymin=poly['shp_data']['ymin']
        ymax=poly['shp_data']['ymax']
        xmin=poly['shp_data']['xmin']
        xmax=poly['shp_data']['xmax']
        pf.write("  <bounds minlat='%0.9f' minlon='%0.9f' maxlat='%0.9f' maxlon='%0.9f' />\n" % (ymin,xmin,ymax,xmax))
    
    #set the nodeid
    nodeid=startnodeid
    
    # extract the polygon details to a file
    for pt in poly['shp_data']['parts'][0]['points'] :
        #for every pair of points write a node
        lat=pt['y']
        lon=pt['x']
        pf.write("  <node id='%d' visible='true' lat='%0.9f' lon='%0.9f' />\n" % (nodeid,lat,lon))
        nodeid-=1
    
    #add the way to add
    wayid=startwayid # ready for multiple ways 
    pf.write("  <way id='%d' action='modify' visible='true'>\n" % (wayid))
    
    #add the list of nodes to the way
    for i in range(startnodeid,nodeid,-1) :
        pf.write("    <nd ref='%d' />\n" % (i))
    
    # add the tags for the way
    pf.write("    <tag k='source' v='OS_Opendata_BoundaryLine' />\n")
    pf.write("    <tag k='natural' v='coastline' />\n")
    
    # end the way and the whole file
    pf.write("  </way>\n")
    pf.write("</osm>\n")
    
    del pf
    return 0

if __name__ == '__main__': main()
